import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import QtQuick.Window 2.12
import org.kde.kwin 3.0 as KWinComponents
import QtGraphicalEffects 1.0
import "Controller.js" as Controller
import IconTheme 1.0

Rectangle {
    id : desktopRect
    x : 0
    y : 0
    width : 100
    height : 100
    color : desktopAreaColor

    property ListModel desktopModel : null
    property var rootRect : null
    property var desktopAreaController  : null

    //虚拟桌面的各个参数
    property double desktopScale : 16/135                                      //虚拟桌面缩放比
    property int desktopThumbnailWidth : rootRect.width * desktopScale        //虚拟桌面缩略图高度
    property int desktopThumbnailHeight : rootRect.height * desktopScale     //虚拟桌面缩略图宽度
    property int desktopCaptionHeight : 24                                  //虚拟桌面标题栏高度
    property int desktopBorderWidth : 4                                    //虚拟桌面边框宽度
    property int desktopRectWidth : desktopThumbnailWidth + 2 * desktopBorderWidth


    ListView {
        id : desktopListView
        y : desktopRect.height / 2 - desktopThumbnailHeight / 2 - 10
        width : parent.width
        height : desktopRect.height
        property bool initMultitask : true
        orientation : ListView.Horizontal
        model : listModel
        spacing : 24
        footerPositioning : ListView.InlineFooter

        NumberAnimation {
            id : desktopListViewLeftMarginAni
            target : desktopListView
            property : "leftMargin"
            duration : 400
            easing.type : Easing.InOutQuad
        }

        add : Transition {
            NumberAnimation {
                properties : "scale,opacity"
                from : 0
                to : 1
                duration : 400
                easing.type : Easing.InOutQuad
            }
        }

        remove : Transition{
            ParallelAnimation{
                NumberAnimation {
                    properties : "scale,opacity"
                    from : 1
                    to : 0
                    duration : 400
                    easing.type : Easing.InOutQuad
                }
            }
        }

        removeDisplaced : Transition {
            NumberAnimation { properties : "x,y"; duration : 1000 }
        }

        move : Transition {
            NumberAnimation {
                property : "x"
                duration : 200
                easing.type : Easing.InOutQuad
            }
        }
        moveDisplaced : Transition {
            NumberAnimation {
                property : "x"
                duration : 200
                easing.type : Easing.InOutQuad
            }
        }
        delegate : Rectangle {
            id : desktopListViewDelegate
            width : desktopThumbnailWidth + 2 * desktopBorderWidth
            height : desktopThumbnailHeight + desktopCaptionHeight + 2 * desktopBorderWidth
            color : "transparent"
            PreviewWindow{
                id : desktopDragRect
                width : desktopListViewDelegate.width
                height : desktopListViewDelegate.height
                dragAxis : Drag.XAxis
                previewAreaWidth : desktopThumbnailWidth + 2 * desktopBorderWidth
                previewAreaHeight : desktopThumbnailHeight + 2 * desktopBorderWidth
                iconVisble : false
                opacity : 1
                wid: value //工作区id会根据工作区的删除交换等操作改变，因此不使用函数赋值
                windowType : "desktop"
                controller : new Controller.DesktopController(multitaskViewModel, desktopDragRect, multitaskViewManager, ["app","desktop"]);
                captionText : "工作区 " + value //工作区名称会根据工作区的删除交换等操作改变，因此不使用函数赋值
                captionColor: desktopCaptionColor
                property var nowDragActive : false

                onWinowDragActiveChanged : {
                    if(winowDragActive)
                    {
                        desktopMaskColor.visible = false
                        opacity = 0.75
                    }
                    else
                    {
                        opacity = 1
                    }

                    if (winowDragActive !== nowDragActive) {
                        nowDragActive = winowDragActive
                    }
                }

                onNowDragActiveChanged : {
                    if(!desktopDragRect.winowDragActive) {
                        desktopDragRect.posX = 0
                        desktopDragRect.posY = 0
                        desktopAreaController.onSourceDropped(desktopDragRect,allroot.highlightDesktop,allroot.highlightDesktop)
                    }
                }

                states :  [
                    State {
                        name : "default"
                        PropertyChanges { target : desktopDragRect; borderWidth : allroot.highlightDesktop == value? 4  : 0 }
                        PropertyChanges { target : desktopDragRect; closeButtonVisble : false }
                    },

                    State {
                        name : "hover"
                        PropertyChanges { target : desktopDragRect; borderWidth : 4 }
                        PropertyChanges { target : desktopDragRect; closeButtonVisble : true }
                    }
                ]

                Rectangle {
                    id: workspace
                    anchors.horizontalCenter : desktopDragRect.horizontalCenter
                    x : desktopBorderWidth
                    y : desktopCaptionHeight+desktopBorderWidth+1
                    width : desktopThumbnailWidth
                    height : desktopThumbnailHeight
                    color : Qt.rgba(255,255,255, 0.5)
                    clip :true
                    visible: true
                    layer.enabled : multitaskViewManager.useOpenGL() ? true : false
                    layer.effect : OpacityMask{
                        maskSource : Rectangle{
                            width : desktopThumbnailWidth
                            height : desktopThumbnailHeight
                            radius : 8
                        }
                    }

                    KWinComponents.DesktopBackground {
                        id : desktopThumbnailBackground
                        anchors.centerIn : parent
                        width : desktopThumbnailWidth
                        height : desktopThumbnailHeight
                        blurAlpha : 0
                    }
                    DropArea {
                        id : previewDropArea
                        anchors.fill : parent

                        onEntered : {
                            desktopMaskColor.visible = true
                            desktopDragRect.controller.onSourceEntered(drag.source,drag.source.wid,desktopDragRect.wid);
                        }

                        onExited : {
                            desktopMaskColor.visible = false
                        }

                        onDropped : {
                            desktopMaskColor.visible = false
                            desktopDragRect.controller.onSourceDropped(drop.source,allroot.highlightDesktop,desktopDragRect.wid);
                            root.updateListModel()
                        }
                    }

                    Repeater {
                        id : desktopWindowRepeater
                        model : listModel.get(index).model

                        KWinComponents.WindowThumbnailItem {
                            id : desktopWindowThumbnail
                            width : model.width * desktopScale
                            height : model.height * desktopScale
                            visible : !(model.minimized)
                            x : (model.x - screenRect.x) * desktopScale
                            y : (model.y - screenRect.y) * desktopScale
                            wId : model.uuid
                            clip : true
                        }
                    }

                    Rectangle {
                        id : desktopMaskColor
                        visible : false
                        anchors.fill: parent
                        color :  desktopMaskReceiveColor
                    }
                }
              }
        }

        onCountChanged : {
            if (initMultitask) {
                if (count === listModel.count) {
                    initMultitask = false;
                }
                return;
            }
            var newWidth = desktopRectWidth * (count + 1) + desktopListView.spacing * (count)
            desktopListViewLeftMarginAni.to = (parent.width - newWidth) / 2 < 0 ? 0  : (parent.width - newWidth) / 2
            desktopListViewLeftMarginAni.start()
        }


        footer : Item {
            id : newDesktopButton
            width : desktopThumbnailWidth + 2 * desktopBorderWidth + 24
            height : desktopThumbnailHeight + desktopCaptionHeight + 2 * desktopBorderWidth
            visible : desktopListView.count < 16 ? true  : false

            PreviewWindow {
                id : spacingRect
                anchors.fill : parent
                anchors.leftMargin : 24
                previewAreaHeight : desktopThumbnailHeight+ 2 * desktopBorderWidth
                previewAreaWidth :desktopThumbnailWidth+ 2 * desktopBorderWidth
                captionColor: desktopCaptionColor
                iconVisble : false
                windowType : "creation"
                controller : new Controller.CreationController(multitaskViewModel, spacingRect, multitaskViewManager, ["app"]);

                DropArea {
                    id : newDesktopDropArea
                    anchors.fill : parent

                    onEntered : {
                        createDesktopButton.color = createDesktopButtonReceiveColor
                    }

                    onExited : {
                        createDesktopButton.color = createDesktopButtonDefaultColor
                    }

                    onDropped : {
                        spacingRect.controller.onSourceDropped(drop.source,allroot.highlightDesktop,listModel.count+1);
                        root.updateListModel()
                    }
                }

                Rectangle {
                    id : createDesktopButton
                    width : desktopThumbnailWidth
                    height : desktopThumbnailHeight
                    x : desktopBorderWidth
                    y : desktopCaptionHeight+desktopBorderWidth
                    radius : 8
                    color : createDesktopButtonDefaultColor

                    Rectangle {
                        anchors.centerIn : parent
                        color : desktopCaptionColor
                        width : newDesktopButton.height / 4
                        height : newDesktopButton.height / 100
                    }

                    Rectangle {
                        anchors.centerIn : parent
                        color : desktopCaptionColor
                        width : newDesktopButton.height / 100
                        height : newDesktopButton.height / 4
                    }
                }

                Component.onCompleted : {
                    spacingRect.setCaption("新建工作区")
                    spacingRect.setId(0)
                }
            }
        } // footer : newdesktopButton
    } // desktopListView
}
