import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import "TabletController.js" as TabletController
import IconTheme 1.0
import org.kde.plasma 2.0 as PlasmaCore
import QtGraphicalEffects 1.0

Rectangle {
    id : tabletPreviewWindow
    width : 498
    height : 320
    color : "transparent"

    property ListModel premodel : null
    property bool closeButtonVisble : false
    property real borderWidth : 0
    property var stateList : ["default", "hover"];
    state : "default"
    property int wid : -1
    //当前预览窗口在桌面上的未缩放前的实际宽度
    property int preViewNoScaledWidth : -1
    //当前预览窗口在桌面上的未缩放前的实际高度
    property int preViewNoScaledHeight : -1
    //当前预览窗口在桌面上的未缩放前的X坐标
    property int preViewNoScaledXCoordinate : -1
     //当前预览窗口在桌面上的未缩放前的Y坐标
    property int preViewNoScaledYCoordinate : -1
    property var captionText  : ""
    property var appController : new TabletController.TabletAppController(multitaskViewModel, appWindow, multitaskViewManager, ["app"])
    property var thumbAnchorRootMapGloableX : mapToGlobal(thumbAnchorRoot.x,thumbAnchorRoot.y).x
    property var thumbAnchorRootMapGloableY : mapToGlobal(thumbAnchorRoot.x,thumbAnchorRoot.y).y
    property int thumbnailRadius: multitaskViewManager.useOpenGL() ? 14 : 0

    /***************************************动画***************************/
    //全部清除窗口的动效
    ParallelAnimation
    {
        id:clearAllWindowAnimation
        NumberAnimation {
            target: tabletPreviewWindow
            properties: "y"
            to: Math.floor(tabletPreviewWindow.y - tabletPreviewWindow.height * 2)
            duration: 300
            easing.type: Easing.InOutQuart
        }

        NumberAnimation {
            target: tabletPreviewWindow
            properties: "opacity"
            to: 0
            duration: 300
            easing.type: Easing.InOutQuart
        }

        onFinished: {
            removeAppWindow(wid)
            if (premodel.count <= 0)
            {
                appController.removeAppWindowOnDesktop()
            }
        }
    }

    //点击空白处关闭多任务视图动效（小窗口逐渐在原地消失）
    NumberAnimation {
        id : spaceClickedPreWindowAnimation
        target : tabletPreviewWindow
        property : "opacity"
        duration : 300
        easing.type : Easing.InOutQuart
        from : 1
        to : 0
        onFinished: {
            qmlViewClosed()
        }
    }

    //! 上滑关闭窗口动效
    NumberAnimation {
        id : slideUpToCloseWindowAnimation
        target : tabletPreviewWindowMove
        property : "y"
        //因为需要上滑完全离开屏幕 尤其第二行预览窗口，故需要-（预览窗口距离屏幕顶端的距离+预览窗口本身高度）
        to : Math.floor(0 - appWindowBack.y- tabletPreviewWindowMove.height)
        duration : 200
        easing.type : Easing.InOutQuart
        onFinished: {
            removePreviewWindow(wid)
        }
    }
    //取消上滑后，恢复到原始位置的动画
    NumberAnimation {
        id : appWindowReturnBackAnimation
        target : tabletPreviewWindow
        property : "y"
        to : Math.floor(tabletPreviewWindowMove.y)
        duration : 200
        easing.type : Easing.InOutQuart
    }

    //点击应用程序预览窗口激活应用程序时的动画特效
    ParallelAnimation{
        id : activateWindowAnimation
        NumberAnimation{
            target : thumbAnchorRoot
            property : "x"
            from : Math.floor(thumbAnchorRootMapGloableX)
            to : preViewNoScaledXCoordinate
            duration : 250
            easing.type : Easing.InOutQuart
        }
        NumberAnimation{
            target : thumbAnchorRoot
            property : "y"
            from : Math.floor(thumbAnchorRootMapGloableY)
            to : preViewNoScaledYCoordinate
            duration : 250
            easing.type : Easing.InOutQuart
        }
        NumberAnimation{
            target : thumbAnchorRoot
            property : "width"
            to : preViewNoScaledWidth
            duration : 250
            easing.type : Easing.InOutQuart
        }
        NumberAnimation{
            target : thumbAnchorRoot
            property : "height"
            to :  preViewNoScaledHeight
            duration : 250
            easing.type : Easing.InOutQuart
        }
        onFinished: {
            qmlViewClosed()
        }
    }


    /***************************************函数*************************/
    function removePreviewWindow(wid)
    {
        appController.removePreviewWindow(wid)
        if ( premodel.count<= 1)
        {
            qmlViewClosed()
        }
        removeAppWindow(wid)
    }

    function removeAppWindow(wid){
        for(var i = 0; i < premodel.count; i++){
            var info = premodel.get(i);
            if(info.id === wid) {
                premodel.remove(i);
                return;
            }
        }
    }

    function appWindowClicked(){
        if (!multitaskViewModel.isTabletMode)
        {
            appController.onWindowClicked(wid);
            qmlViewClosed()
        }
        else
        {
            thumbAnchorRootMapGloableY = Math.floor(mapToGlobal(thumbMove.x,thumbMove.y).y)
            thumbAnchorRootMapGloableX = Math.floor(mapToGlobal(thumbMove.x,thumbMove.y).x)
            thumbAnchorRoot.thumbClicked = true
            captionRow.visible = false
            thumbBorder.border.width  = 0
            preWindowCloseButton.visible = false
            appController.onWindowClicked(wid);
        }
    }

    function setStatus(status) {
        if(stateList.indexOf(status) < 0)
        {
            return;
        }
        if (status === state) {
            return
        }
        state = status;
    }

    function setDefaultStatus() {
        setStatus("default")
    }

    function setHoverStatus() {
        setStatus("hover")
    }

    function startClearAllWindowAnimation()
    {
        clearAllWindowAnimation.start()
    }

    function startSpaceClickedPreWindowAnimation()
    {
        spaceClickedPreWindowAnimation.start()
    }

    function startCloseWindowAnimation()
    {
        slideUpToCloseWindowAnimation.start()
    }

    function startResumeWindowAnimation()
    {
        appWindowReturnBackAnimation.start()
    }

    function startActiveWindowAnimation()
    {
        activateWindowAnimation.start()
    }

    function setCaption(titleText) {
        caption.text = titleText;
    }

    function setId(wId) {
        wid = wId;
    }

    function setPreViewNoScaledWidth(Width) {
        preViewNoScaledWidth = Width;
    }

    function setPreViewNoScaledHeight(Height) {
        preViewNoScaledHeight = Height;
    }

    function setPreViewNoScaledXCoordinate(X) {
        preViewNoScaledXCoordinate = X;
    }

    function setPreViewNoScaledYCoordinate(Y) {
        preViewNoScaledYCoordinate = Y;
    }

    //控制白色边框是否显示
    MouseArea{
        id : preWindowMouseArea
        anchors.fill : parent
        preventStealing : true
        hoverEnabled : true

        onEntered : {
            appController.setHoverStatus();
        }

        onExited: {
            if(!preWindowCloseButton.closeButtonContainMouse)
            {
                appController.setDefaultStatus();
            }
        }

        onClicked: {
            appWindowClicked()
        }
    }

    Rectangle{
        id : tabletPreviewWindowMove
        width : Math.floor(parent.width)
        height : Math.floor(parent.height)
        color : "transparent"
        radius : tabletPreviewWindow.thumbnailRadius
        ColumnLayout {
            id : columnLayout
            spacing : iconToThumbSpacing
            anchors.centerIn : parent
            Layout.fillHeight : true
            width : Math.floor(tabletPreviewWindow.width)

            RowLayout {
                id : captionRow
                spacing : 12 * tabletScaleFactor
                Layout.fillHeight : true

                Rectangle {
                    id : iconItem
                    height : Math.floor(iconAndCloseButtonHeight)
                    width : Math.floor(iconAndCloseButtonHeight)
                    color : "transparent"
                    IconTheme{
                        anchors.fill : parent
                        icon : multitaskViewModel.getWindowIcon(wid)
                    }
                }

                Label {
                    id : caption
                    Layout.fillWidth : true
                    text : captionText
                    fontSizeMode : Text.HorizontalFit
                    elide : Text.ElideRight
                    color : "#FFFFFF"
                }

                TabletViewCloseButton {
                    id : preWindowCloseButton

                    states :  [
                        State {
                            name : "default"
                            PropertyChanges { target : preWindowCloseButton; imageSource : "qrc:/images/ukui-taskview-close-default.svg" }
                        },
                        State {
                            name : "hover"
                            PropertyChanges { target : preWindowCloseButton; imageSource : "qrc:/images/ukui-taskview-close-hover.svg" }
                        },
                        State {
                            name : "press"
                            PropertyChanges { target : preWindowCloseButton; imageSource : "qrc:/images/ukui-taskview-close-click.svg" }
                        }
                    ]

                    Component.onCompleted: {
                        preWindowCloseButton.setId(model.id)
                    }
                }
            }//RowLayout captionRow

            Rectangle{
                id : thumbBorder
                height : Math.floor(preWinHeight)
                width : Math.floor(preWinWidth + 8)
                color : "transparent"
                radius : tabletPreviewWindow.thumbnailRadius
                border.color : "#FFFFFF"
                border.width : Math.floor(borderWidth)

                Rectangle{
                    id : thumbAnchorRoot
                    height : Math.floor(preWinHeight)
                    width : Math.floor(preWinWidth)
                    color : Qt.rgba(255,255,255, 0.5)
                    x : 4
                    y : 0
                    radius : tabletPreviewWindow.thumbnailRadius
                    property bool thumbClicked: false//记录是否是点击缩略图

                    Rectangle{
                        id : thumbMove
                        width : Math.floor(parent.width)
                        height : Math.floor(parent.height)
                        color : "transparent"
                        anchors.centerIn : parent
                        radius : tabletPreviewWindow.thumbnailRadius

                        states: [
                            State {
                                name : "mousestate"
                                when : thumbAnchorRoot.thumbClicked
                                ParentChange {
                                    target : thumbAnchorRoot
                                    parent : rootView
                                }
                            }
                        ]

                        MouseArea{
                            anchors.fill : parent
                            //这里没有鼠标透传的原因：如果鼠标透传的话，会影响Scrollview的拖拽功能
                            onClicked: {
                                appWindowClicked()
                            }
                        }

                        Rectangle {
                            id:thumb
                            visible:true

                            function thumbWidth(){
                                var factor = model.height / thumbMove.height;
                                var width = model.width / factor;
                                return Math.min(thumbMove.width, width);
                            }

                            function thumbHeight(){
                                var factor = thumbMove.width * model.height
                                var height = factor / model.width;
                                return Math.floor(Math.min(thumbMove.height, height))
                            }

                            width : Math.floor(thumb.thumbWidth())
                            height : Math.floor(thumb.thumbHeight())
                            anchors.centerIn : parent
                            radius : tabletPreviewWindow.thumbnailRadius
                            color : "transparent"
                            clip : true
                            layer.enabled : multitaskViewManager.useOpenGL() ? true : false
                            layer.effect : OpacityMask{
                                maskSource : Rectangle{
                                    width : Math.floor(thumb.width)
                                    height : Math.floor(thumb.height)
                                    radius : tabletPreviewWindow.thumbnailRadius
                                }
                            }
                            PlasmaCore.WindowThumbnail{
                                anchors.centerIn : parent
                                height : Math.floor(thumb.height)
                                width : Math.floor(thumb.width)
                                winId : wid
                                clip : true
                            }
                        }
                    }
                }
            }
        }
    }
    DragHandler{
        property int dragTime : 0
        property var beginTime : Date.now()
        id : windowDrag
        target : tabletPreviewWindow
        xAxis.enabled : false
        yAxis.maximum : 0

        onActiveChanged: {
            if (active){
                beginTime = Date.now()
            }
            if (!active){
                var subTime = Date.now() - beginTime
                var subDistance = Math.abs(tabletPreviewWindowMove.y - tabletPreviewWindow.y)
                var v = Math.abs(subDistance) / subTime;
                console.log("移动距离 ，窗口的三分之二" + subDistance ,  tabletPreviewWindow.height * 1 / 2)
                console.log("速度为: " + v)
                // 平均速度或者位移距离
                if (v > 0.5 || subDistance >= tabletPreviewWindow.height * 1 / 2){
                    appController.slideUpCloseWindow()
                }else{
                    appController.startResumeWindowAnimation()
                }
            }
        }
    }//windowDrag
}


