import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import "TabletController.js" as TabletController
import IconTheme 1.0
import org.kde.plasma 2.0 as PlasmaCore
import QtGraphicalEffects 1.0

Rectangle {
    id : closeButton
    visible : closeButtonVisble
    width : Math.floor(iconAndCloseButtonHeight)
    height : Math.floor(iconAndCloseButtonHeight)
    radius : 14
    color : "transparent"

    property var closeButtonController : new TabletController.CloseButtonController(multitaskViewModel, closeButton, multitaskViewManager, ["app"])
    property var buttonStateList : ["default", "hover", "press"];
    state : "default"
    property var imageSource : "qrc:/images/ukui-taskview-close-default.svg"
    property int wid : -1
    property alias closeButtonContainMouse: closeButtonMouseArea.containsMouse

    Image {
        id: closeWindow
        anchors.fill : parent
        source : imageSource
        sourceSize.width: Math.floor(parent.width)
        sourceSize.height: Math.floor(parent.height)
    }

    function setStatus(status) {
          if(buttonStateList.indexOf(status) < 0)
          {
              return;
          }
         if (status === state) {
             return
         } else {
             state = status;
         }
    }

    function setDefaultStatus() {
        setStatus("default")
    }

    function setHoverStatus() {
        setStatus("hover")
    }

    function setPressStatus() {
        setStatus("press")
    }

    function setId(wId) {
        wid = wId;
    }

    MouseArea{
        id : closeButtonMouseArea
        anchors.fill : parent
        hoverEnabled : true
        onEntered: {
           closeButtonController.setHoverStatus()
        }

        onExited: {
            closeButtonController.setDefaultStatus()
            if(!preWindowMouseArea.containsMouse)
            {
                tabletPreviewWindow.setDefaultStatus()
            }
        }

        onPressed: {
           closeButtonController.setPressStatus()
        }

        onClicked: {
            tabletPreviewWindow.removePreviewWindow(wid)
        }
    }
}
