import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import QtQuick.Window 2.12
import org.kde.kwin 3.0 as KWinComponents
import QtGraphicalEffects 1.0
import "Controller.js" as Controller
import IconTheme 1.0

Repeater {
    id : allroot
    property int screencnt : multitaskViewModel.logicalScreenCount
    model : screencnt
    signal qmlAppWindowCreated(int visiblestatus)
    signal qmlAppWindowRemoved(int Id)
    signal qmlAppWindowActivated()
    signal qmlAppWindowDesktopChanged(int windowId, int oldDesktopIndex, int newDesktopIndex)
    signal qmlDesktopRemoved(int Id)
    signal qmlDesktopAppended(int Id)
    signal qmlDesktopMoved(int oldDesktop,int newDesktop)
    signal qmlCurrentDesktopChanged(int old,int newId)
    signal qmlScreenCountChanged(int oldCount,int newCount)
    signal qmlScreenSizeChanged()
    property int highlightDesktop : multitaskViewModel.getCurrentDesktop()
    property var allcontroller : new Controller.Controller(multitaskViewModel, allroot, multitaskViewManager, ["none"])
    property var currentTheme  : multitaskViewModel.getUKUIStyleName()
    property var closeButtonHoverColor : setItemColor()
    property var closeButtonDefaultColor : ""
    property var closeButtonClickColor : ""
    property var createDesktopButtonReceiveColor : ""
    property var createDesktopButtonDefaultColor : ""
    property var desktopMaskReceiveColor : ""
    property var desktopCaptionColor : ""
    property var desktopAreaColor : ""
    focus : true

    function setItemColor() {
        closeButtonDefaultColor = Qt.rgba(255,255,255, 0.15)
        if (allroot.currentTheme === "ukui-default" || allroot.currentTheme === "ukui-white"
                || allroot.currentTheme === "ukui" || allroot.currentTheme === "ukui-light")
        {
            closeButtonHoverColor = Qt.rgba(255,255,255, 1)
            closeButtonClickColor = Qt.rgba(217,217,217, 1)
            createDesktopButtonReceiveColor = Qt.rgba(0, 0, 0, 0.25)
            createDesktopButtonDefaultColor = Qt.rgba(0, 0, 0, 0.15)
            desktopMaskReceiveColor = Qt.rgba(0, 0, 0, 0.25)
            desktopCaptionColor = Qt.rgba(0,0,0,1)
            desktopAreaColor = Qt.rgba(245/255, 245/255, 245/255,0.75)
        }
        else
        {
            closeButtonHoverColor =  "#37373B"
            closeButtonClickColor = Qt.rgba(75,75,79, 1)
            createDesktopButtonReceiveColor = Qt.rgba(0, 0, 0, 0.1)
            createDesktopButtonDefaultColor = Qt.rgba(255, 255, 255, 0.15)
            desktopMaskReceiveColor = Qt.rgba(255, 255, 255, 0.25)
            desktopCaptionColor = Qt.rgba(255, 255, 255, 1)
            desktopAreaColor = Qt.rgba(35/255, 36/255, 38/255, 0.75)
        }
    }

    onQmlAppWindowCreated : {
        // allroot.allcontroller.closeMultitaskView();
    }

    onQmlAppWindowActivated: {
        allroot.allcontroller.closeMultitaskView();
    }

    onQmlScreenCountChanged : {
        allroot.allcontroller.closeMultitaskView();
    }

    onQmlScreenSizeChanged : {
        allroot.allcontroller.closeMultitaskView();
    }

    Keys.onPressed : {
        if (event.key === Qt.Key_Escape) {
            allroot.allcontroller.closeMultitaskView();
            return;
        }
        if (event.modifiers & Qt.MetaModifier)
        {
            if (event.key === Qt.Key_Tab)
            {
                allroot.allcontroller.closeMultitaskView();
                return;
            }
        }
    }

    Rectangle {
        // 窗口缩略图区域
        id :root
        property var windowIndex : index
        property var screenRect : multitaskViewModel.getFullScreenGeometry(multitaskViewModel.getCurrentDesktop(),root.windowIndex)
        width : screenRect.width
        height : screenRect.height
        x : screenRect.x
        y : screenRect.y

        color : "transparent"
        objectName : "root"
        property var controller : new Controller.Controller(multitaskViewModel, root, multitaskViewManager, ["none"])

        Connections{
            target : allroot
            onQmlAppWindowRemoved : {
                for(var i = 0;i<myModel.count;i++){
                    var info = myModel.get(i);
                    if(info.id === Id) {
                        myModel.remove(i);
                        root.updateListModel()
                        return;
                    }
                }
            }

            onQmlAppWindowDesktopChanged : {
                appRemoved(windowId);
            }

            onQmlCurrentDesktopChanged : {
                root.resetModel(newId)
                allroot.highlightDesktop = newId
            }

            onQmlDesktopRemoved : {
                // 移除指定虚拟桌面，并更新虚拟桌面索引值
                listModel.remove(Id - 1);
                for(var i = Id - 1; i < listModel.count; ++i)
                {
                    var object = listModel.get(i);
                    object.value = i+1
                }
                // 更新小窗口
                root.updateListModel()

                if (Id !== 1) {
                    Id -= 1
                }

                multitaskViewModel.setCurrentDesktop(Id);
                allroot.highlightDesktop  = multitaskViewModel.getCurrentDesktop()
                root.resetModel(multitaskViewModel.getCurrentDesktop())
            }

            onQmlDesktopAppended : {
                // listmodel中增加一个虚拟桌面
                listModel.append({value : Id, model : []});
                multitaskViewModel.setCurrentDesktop(Id);
                allroot.highlightDesktop  = multitaskViewModel.getCurrentDesktop()
            }

            onQmlDesktopMoved : {
                // 交换listModel中的指定两项
                listModel.move(oldDesktop-1,newDesktop-1 , 1);
                multitaskViewModel.setCurrentDesktop(newDesktop);
                allroot.highlightDesktop = newDesktop
                // 交换后更新索引值
                for (var i = oldDesktop < newDesktop ? oldDesktop : newDesktop; i <= listModel.count; ++i)
                {
                    var object = listModel.get(i-1);
                    object.value = i;
                }
            }
        }

        KWinComponents.DesktopBackground {
            id : wholeDesktopBackground
            width : root.width
            height : root.height
            blurAlpha : 0.7
        }

        property ListModel myModel : ListModel{}
        Component.onCompleted : {
            myModel.clear();
            var windowInfo = multitaskViewModel.getAppWindowList(multitaskViewModel.getCurrentDesktop(),root.windowIndex)
            for(var i in windowInfo)
            {
                var info = windowInfo[i]
                myModel.append({"x" : info.x, "y" : info.y,
                                   "width" : info.width, "height" : info.height,
                                   "id" : info.id,"minimized" :info.minimized, "caption" : info.caption,
                                   "uuid" : String(info.uuid)});
            }
        }

        function resetModel(deskindex) {
            var windowInfo = multitaskViewModel.getAppWindowList(deskindex,windowIndex)
            myModel.clear();
            for(var i in windowInfo)
            {
                var info = windowInfo[i]
                myModel.append({"x" : info.x, "y" : info.y,
                                   "width" : info.width, "height" : info.height,
                                   "id" : info.id,"minimized" : info.minimized, "caption" : info.caption,
                                   "uuid" : String(info.uuid)});
            }
        }

        function appRemoved(wid){
            for(var i = 0;i<myModel.count;i++){
                var info = myModel.get(i);
                if(info.id === wid) {
                    myModel.remove(i);
                    return;
                }
            }
        }

        AppArea{
            id : appArea
            width : parent.width
            height : parent.height-desktopArea.height
            appModel : myModel
            rootRect : screenRect
            appAreaController : new Controller.Controller(multitaskViewModel, appArea, multitaskViewManager, ["none"])
        }
        //虚拟桌面

        ListModel {
            id: listModel

            Component.onCompleted: {
                resetListModel();
            }
        }

        function resetListModel(){
            listModel.clear()
            var desktopinfo = multitaskViewModel.getDesktopList();
            // 重置listModel信息
            for (var i in desktopinfo)
            {
                // 获取虚拟桌面上的窗口显示情况
                var winModel = [];
                var infos = desktopinfo[i]
                var screenlist =infos.screenList
                var appList = screenlist[root.windowIndex].windowList
                for(var j in appList)
                {
                    var info = appList[j]

                    winModel.push({"x": info.x, "y": info.y,
                                      "width": info.width, "height": info.height,
                                      "id": info.id,"minimized":info.minimized, "caption": info.caption,
                                      "uuid" : String(info.uuid)});
                }
                listModel.append({ value: infos.index, model: winModel});
            }
        }

        function updateListModel(){
            // 更新全部listModel信息
            var desktopinfo = multitaskViewModel.getDesktopList();
            for (var i in desktopinfo)
            {
                listModel.get(i).model.clear();
                var infos = desktopinfo[i]
                var screenlist =infos.screenList
                var appList = screenlist[root.windowIndex].windowList
                for(var j in appList)
                {
                    var info = appList[j]
                    listModel.get(i).model.append({"x": info.x, "y": info.y,
                                                      "width": info.width, "height": info.height,
                                                      "id": info.id,"minimized":info.minimized, "caption": info.caption,
                                                      "uuid" : String(info.uuid)});
                }
            }
        }

        DesktopArea {
            id :desktopArea
            x : 0
            y : parent.height * 111/135
            width : parent.width
            height : parent.height - y
            desktopModel : listModel
            rootRect : screenRect
            desktopAreaController : new Controller.Controller(multitaskViewModel, desktopArea, multitaskViewManager, ["none"])
        }
    }
}
